#include "../../src/Library/CreatureData/CreatureSpecies.hpp"
#include "../Core.hpp"
using namespace CreatureLib::Library;

export uint8_t CreatureLib_CreatureSpecies_Construct(CreatureSpecies*& out, uint16_t id, const char* name,
                                                     SpeciesVariant* defaultVariant, float genderRatio,
                                                     const char* growthRate, uint8_t captureRate, const char* flags[],
                                                     size_t flagsCount) {

    Try(std::unordered_set<uint32_t> conversedFlags(flagsCount);
        for (size_t i = 0; i < flagsCount; i++) { conversedFlags.insert(ArbUt::StringView::CalculateHash(flags[i])); }

        out = new CreatureSpecies(id, ArbUt::StringView(name), defaultVariant, genderRatio,
                                  ArbUt::StringView(growthRate), captureRate, conversedFlags);)
}

export void CreatureLib_CreatureSpecies_Destruct(const CreatureSpecies* p) { delete p; }

SIMPLE_GET_FUNC(CreatureSpecies, GetId, uint16_t);
SIMPLE_GET_FUNC(CreatureSpecies, GetGenderRate, float);
SIMPLE_GET_FUNC(CreatureSpecies, GetCaptureRate, uint8_t);
export const char* CreatureLib_CreatureSpecies_GetName(const CreatureSpecies* p) { return p->GetName().c_str(); }
export const char* CreatureLib_CreatureSpecies_GetGrowthRate(const CreatureSpecies* p) {
    return p->GetGrowthRate().c_str();
}
export bool CreatureLib_CreatureSpecies_HasVariant(const CreatureSpecies* p, const char* name) {
    return p->HasVariant(ArbUt::StringView::CalculateHash(name));
}
export bool CreatureLib_CreatureSpecies_HasVariantWithHash(const CreatureSpecies* p, uint32_t hash) {
    return p->HasVariant(hash);
}
export bool CreatureLib_CreatureSpecies_TryGetVariant(const CreatureSpecies* p, const char* name,
                                                      const SpeciesVariant*& out) {
    ArbUt::BorrowedPtr<const SpeciesVariant> o;
    auto res = p->TryGetVariant(ArbUt::StringView::CalculateHash(name), o);
    out = o.GetRaw();
    return res;
}
export bool CreatureLib_CreatureSpecies_TryGetVariantWithHash(const CreatureSpecies* p, uint32_t hash,
                                                              const SpeciesVariant*& out) {
    ArbUt::BorrowedPtr<const SpeciesVariant> o;
    auto res = p->TryGetVariant(hash, o);
    out = o.GetRaw();
    return res;
}

export uint8_t CreatureLib_CreatureSpecies_GetVariant(const SpeciesVariant*& out, const CreatureSpecies* p,
                                                      const char* name) {
    Try(out = p->GetVariant(ArbUt::StringView::CalculateHash(name)).GetRaw();)
}
export uint8_t CreatureLib_CreatureSpecies_GetVariantWithHash(const SpeciesVariant*& out, const CreatureSpecies* p,
                                                              uint32_t hash) {
    Try(out = p->GetVariant(hash).GetRaw();)
}
export uint8_t CreatureLib_CreatureSpecies_SetVariant(CreatureSpecies* p, const char* name, SpeciesVariant* variant) {
    Try(p->SetVariant(ArbUt::StringView(name), variant);)
}

export Gender CreatureLib_CreatureSpecies_GetRandomGender(CreatureSpecies* p, ArbUt::Random* random) {
    return p->GetRandomGender(*random);
}

export size_t CreatureLib_CreatureSpecies_GetVariantsCount(CreatureSpecies* p) {
    return p->GetVariantsIterator().Count();
}
export const SpeciesVariant* const* CreatureLib_CreatureSpecies_GetVariants(CreatureSpecies* p) {
    return reinterpret_cast<const SpeciesVariant* const*>(p->GetVariantsIterator().RawData());
}
export bool CreatureLib_CreatureSpecies_HasFlag(const CreatureSpecies* p, const char* key) {
    return p->HasFlag(ArbUt::StringView::CalculateHash(key));
}