#include "DamageLibrary.hpp"
#include "../ScriptHandling/ScriptMacros.hpp"

using namespace CreatureLib::Battling;
uint32_t DamageLibrary::GetDamage(ExecutingAttack* attack, Creature* target, uint8_t hitIndex,
                                  const ExecutingAttack::HitData& hitData) const {
    AssertNotNull(attack)
    AssertNotNull(target)
    auto levelMod = static_cast<float>(2 * attack->GetUser()->GetLevel()) / 5 + 2;
    auto bp = hitData.GetBasePower();
    auto statMod = GetStatModifier(attack, target, hitIndex, hitData);
    HOOK(ModifyStatModifier, attack, attack, target, hitIndex, &statMod);
    uint32_t damage = static_cast<uint32_t>((((levelMod * static_cast<float>(bp) * statMod) / 50) + 2) *
                                            GetDamageModifier(attack, target, hitIndex, hitData));
    HOOK(OverrideDamage, attack, attack, target, hitIndex, &damage);
    return damage;
}

uint8_t DamageLibrary::GetBasePower(ExecutingAttack* attack, Creature* target, uint8_t hitIndex,
                                    [[maybe_unused]] const ExecutingAttack::HitData& hitData) const {
    AssertNotNull(attack)
    AssertNotNull(target)
    auto bp = attack->GetAttack()->GetAttack()->GetBasePower();
    HOOK(OverrideBasePower, attack, attack, target, hitIndex, &bp);
    return bp;
}

float DamageLibrary::GetStatModifier(ExecutingAttack* attack, Creature* target, uint8_t hitIndex,
                                     const ExecutingAttack::HitData& hitData) const {
    AssertNotNull(attack)
    AssertNotNull(target)
    auto user = attack->GetUser().GetRaw();
    AssertNotNull(user)
    HOOK(ChangeDamageStatsUser, attack, attack, target, hitIndex, &user);
    Library::Statistic offensiveStat;
    Library::Statistic defensiveStat;
    if (attack->GetAttack()->GetAttack()->GetCategory() == Library::AttackCategory::Physical) {
        offensiveStat = Library::Statistic::PhysicalAttack;
        defensiveStat = Library::Statistic::PhysicalDefense;
    } else {
        offensiveStat = Library::Statistic::MagicalAttack;
        defensiveStat = Library::Statistic::MagicalDefense;
    }

    auto bypassDefensive = hitData.IsCritical() && target->GetStatBoost(defensiveStat) > 0;
    HOOK(BypassDefensiveStat, attack, attack, target, hitIndex, &bypassDefensive);
    auto bypassOffensive = hitData.IsCritical() && user->GetStatBoost(offensiveStat) < 0;
    HOOK(BypassOffensiveStat, attack, attack, target, hitIndex, &bypassOffensive);

    float offensiveValue;
    float defensiveValue;

    if (bypassOffensive) {
        offensiveValue = user->GetFlatStat(offensiveStat);
    } else {
        offensiveValue = user->GetBoostedStat(offensiveStat);
    }
    if (bypassDefensive) {
        defensiveValue = target->GetFlatStat(defensiveStat);
    } else {
        defensiveValue = target->GetBoostedStat(defensiveStat);
    }

    return offensiveValue / defensiveValue;
}

float DamageLibrary::GetDamageModifier(ExecutingAttack* attack, Creature* target, uint8_t hitIndex,
                                       const ExecutingAttack::HitData& hitData) const {
    AssertNotNull(attack)
    AssertNotNull(target)
    float mod = 1;
    mod *= hitData.GetEffectiveness();
    HOOK(ModifyDamageModifier, attack, attack, target, hitIndex, &mod);
    return mod;
}
