#ifndef CREATURELIB_SCRIPT_HPP
#define CREATURELIB_SCRIPT_HPP

#include "../../Library/EffectParameter.hpp"

namespace CreatureLib::Battling {
    class BaseTurnChoice;
    class AttackTurnChoice;
    class SwitchTurnChoice;
    class ExecutingAttack;
    class Creature;

    class Script {
    public:
        virtual ~Script() = default;

        virtual void Stack(){};
        virtual void OnRemove(){};

        virtual const ArbUt::StringView& GetName() const noexcept = 0;

        virtual void
        OnInitialize([[maybe_unused]] const ArbUt::List<CreatureLib::Library::EffectParameter*>& parameters){};
        virtual void OnBeforeTurn([[maybe_unused]] const BaseTurnChoice* choice){};

        virtual void ChangePriority([[maybe_unused]] AttackTurnChoice* choice, [[maybe_unused]] int8_t* priority){};
        virtual void ChangeAttack([[maybe_unused]] AttackTurnChoice* choice,
                                  [[maybe_unused]] ArbUt::StringView* outAttack){};
        virtual void PreventAttack([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] bool* outResult){};
        virtual void FailAttack([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] bool* outFailed){};
        virtual void StopBeforeAttack([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] bool* outResult){};
        virtual void OnBeforeAttack([[maybe_unused]] ExecutingAttack* attack){};

        virtual void FailIncomingAttack([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                        [[maybe_unused]] bool* outResult){};
        virtual void IsInvulnerable([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                    [[maybe_unused]] bool* outResult){};
        virtual void OnAttackMiss([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target){};
        virtual void ChangeAttackType([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                      [[maybe_unused]] uint8_t hitNumber, [[maybe_unused]] uint8_t* outType){};
        virtual void ChangeEffectiveness([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                         [[maybe_unused]] uint8_t hitNumber, [[maybe_unused]] float* effectiveness){};

        virtual void OverrideBasePower([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                       [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] uint8_t* basePower){};
        virtual void ChangeDamageStatsUser([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                           [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] Creature** statsUser){};

        virtual void BypassDefensiveStat([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                         [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] bool* bypass){};
        virtual void BypassOffensiveStat([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                         [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] bool* bypass){};
        virtual void ModifyStatModifier([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                        [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] float* modifier){};
        virtual void ModifyDamageModifier([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                          [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] float* modifier){};
        virtual void OverrideDamage([[maybe_unused]] ExecutingAttack* attack, [[maybe_unused]] Creature* target,
                                    [[maybe_unused]] uint8_t hitIndex, [[maybe_unused]] uint32_t* damage){};

        virtual void PreventSecondaryEffects([[maybe_unused]] const ExecutingAttack* attack,
                                             [[maybe_unused]] Creature* target, [[maybe_unused]] uint8_t hitNumber,
                                             [[maybe_unused]] bool* outResult){};
        virtual void OnSecondaryEffect([[maybe_unused]] const ExecutingAttack* attack,
                                       [[maybe_unused]] Creature* target, [[maybe_unused]] uint8_t hitNumber){};

        virtual void OnAfterHits([[maybe_unused]] const ExecutingAttack* attack, [[maybe_unused]] Creature* target){};

        virtual void PreventSelfSwitch([[maybe_unused]] const SwitchTurnChoice* choice,
                                       [[maybe_unused]] bool* outResult){};

        virtual void ModifyEffectChance([[maybe_unused]] const ExecutingAttack* attack,
                                        [[maybe_unused]] Creature* target, [[maybe_unused]] float* chance){};
        virtual void ModifyIncomingEffectChance([[maybe_unused]] const ExecutingAttack* attack,
                                                [[maybe_unused]] Creature* target, [[maybe_unused]] float* chance){};
    };
}

#endif // CREATURELIB_SCRIPT_HPP
