#ifndef CREATURELIB_DATALIBRARY_HPP
#define CREATURELIB_DATALIBRARY_HPP

#include "AttackLibrary.hpp"
#include "GrowthRates/GrowthRateLibrary.hpp"
#include "ItemLibrary.hpp"
#include "LibrarySettings.hpp"
#include "SpeciesLibrary.hpp"
#include "TypeLibrary.hpp"

namespace CreatureLib::Library {
    /*!
\brief The core library. This library holds all static data for a creature set.
 */
    class DataLibrary {
    private:
        std::unique_ptr<const LibrarySettings> _settings;
        std::unique_ptr<const SpeciesLibrary> _species;
        std::unique_ptr<const AttackLibrary> _attacks;
        std::unique_ptr<const ItemLibrary> _items;
        std::unique_ptr<const GrowthRateLibrary> _growthRates;
        std::unique_ptr<const TypeLibrary> _typeLibrary;

    public:
        DataLibrary(LibrarySettings* settings, CreatureLib::Library::SpeciesLibrary* species,
                    CreatureLib::Library::AttackLibrary* attacks, CreatureLib::Library::ItemLibrary* items,
                    CreatureLib::Library::GrowthRateLibrary* growthRates, TypeLibrary* typeLibrary);

        virtual ~DataLibrary() {}

        [[nodiscard]] inline const std::unique_ptr<const LibrarySettings>& GetSettings() const noexcept {
            return _settings;
        }
        [[nodiscard]] inline const std::unique_ptr<const SpeciesLibrary>& GetSpeciesLibrary() const noexcept {
            return _species;
        }
        [[nodiscard]] inline const std::unique_ptr<const AttackLibrary>& GetAttackLibrary() const noexcept {
            return _attacks;
        }
        [[nodiscard]] inline const std::unique_ptr<const ItemLibrary>& GetItemLibrary() const noexcept {
            return _items;
        }
        [[nodiscard]] inline const std::unique_ptr<const GrowthRateLibrary>& GetGrowthRates() const noexcept {
            return _growthRates;
        }
        [[nodiscard]] inline const std::unique_ptr<const TypeLibrary>& GetTypeLibrary() const noexcept {
            return _typeLibrary;
        }
    };
}

#endif // CREATURELIB_DATALIBRARY_HPP
