#ifdef TESTS_BUILD

#include "../../extern/doctest.hpp"
#include "../../src/Battling/Flow/TurnOrdering.hpp"
#include "../../src/Battling/TurnChoices/AttackTurnChoice.hpp"
#include "../../src/Battling/TurnChoices/PassTurnChoice.hpp"
#include "../TestLibrary/TestLibrary.hpp"

using namespace CreatureLib;
using namespace CreatureLib::Battling;

TEST_CASE("Turn ordering: Attack before pass") {
    auto lib = TestLibrary::Get();
    auto learnedAttack =
        LearnedAttack(lib->GetAttackLibrary()->Get("standard"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto choice1 = std::make_shared<PassTurnChoice>(nullptr);
    auto choice2 = std::make_shared<AttackTurnChoice>(nullptr, &learnedAttack, CreatureIndex(0, 0));
    auto vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice1, choice2};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice2, choice1};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
}

TEST_CASE("Turn ordering: High priority goes before no priority") {
    const auto& l = TestLibrary::Get()->GetAttackLibrary();
    auto a1 = new LearnedAttack(l->Get("standard"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto a2 = new LearnedAttack(l->Get("highPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto choice1 = std::make_shared<AttackTurnChoice>(nullptr, a1, CreatureIndex(0, 0));
    auto choice2 = std::make_shared<AttackTurnChoice>(nullptr, a2, CreatureIndex(0, 0));
    auto vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice1, choice2};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice2, choice1};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);

    delete a1;
    delete a2;
}

TEST_CASE("Turn ordering: Higher priority goes before high priority") {
    const auto& l = TestLibrary::Get()->GetAttackLibrary();
    auto a1 = new LearnedAttack(l->Get("highPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto a2 = new LearnedAttack(l->Get("higherPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto choice1 = std::make_shared<AttackTurnChoice>(nullptr, a1, CreatureIndex(0, 0));
    auto choice2 = std::make_shared<AttackTurnChoice>(nullptr, a2, CreatureIndex(0, 0));
    auto vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice1, choice2};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice2, choice1};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    delete a1;
    delete a2;
}

TEST_CASE("Turn ordering: High priority goes before low priority") {
    const auto& l = TestLibrary::Get()->GetAttackLibrary();
    auto a1 = new LearnedAttack(l->Get("lowPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto a2 = new LearnedAttack(l->Get("higherPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto choice1 = std::make_shared<AttackTurnChoice>(nullptr, a1, CreatureIndex(0, 0));
    auto choice2 = std::make_shared<AttackTurnChoice>(nullptr, a2, CreatureIndex(0, 0));
    auto vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice1, choice2};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice2, choice1};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);

    delete a1;
    delete a2;
}

TEST_CASE("Turn ordering: No priority goes before low priority") {
    const auto& l = TestLibrary::Get()->GetAttackLibrary();
    auto a1 = new LearnedAttack(l->Get("lowPriority"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto a2 = new LearnedAttack(l->Get("standard"_cnc.GetHash()), AttackLearnMethod::Unknown);
    auto choice1 = std::make_shared<AttackTurnChoice>(nullptr, a1, CreatureIndex(0, 0));
    auto choice2 = std::make_shared<AttackTurnChoice>(nullptr, a2, CreatureIndex(0, 0));
    auto vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice1, choice2};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);
    vec = std::vector<std::shared_ptr<BaseTurnChoice>>{choice2, choice1};
    TurnOrdering::OrderChoices(vec);
    CHECK(vec[0] == choice2);
    CHECK(vec[1] == choice1);

    delete a1;
    delete a2;
}

#endif